'From Squeak3.1beta of 3 October 2001 [latest update: #4411] on 3 November 2001 at 10:45:34 am'!
"Change Set:		SlightProgress-sm
Date:			3 Nov 2001
Author:			Simon Michael

Make project loading progress indicators slightly more accurate by storing timing data separately for local & remote downloads.
After loading a few projects you should notice less flickering, especially when downloading large projects over a slow link.
Also fixes the spelling of the TranslucentProgressMorph class."!

Object subclass: #ProjectLoading
	instanceVariableNames: ''
	classVariableNames: 'HistoryVariant '
	poolDictionaries: ''
	category: 'System-Support'!
Smalltalk renameClassNamed: #TranslucentProgessMorph as: #TranslucentProgressMorph!

!ComplexProgressIndicator methodsFor: 'as yet unclassified' stamp: 'sm 5/6/2001 20:43'!
withProgressDo: aBlock 
	| safetyFactor totals trialRect delta stageCompletedString |
	Smalltalk isMorphic
		ifFalse: [^ aBlock value].
	formerProject _ Project current.
	formerWorld _ World.
	formerProcess _ Processor activeProcess.
	targetMorph
		ifNil: [targetMorph _ ProgressTargetRequestNotification signal].
	targetMorph
		ifNil: [trialRect _ Rectangle center: Sensor cursorPoint extent: 80 @ 80.
			delta _ trialRect amountToTranslateWithin: formerWorld bounds.
			trialRect _ trialRect translateBy: delta.
			translucentMorph _ TranslucentProgressMorph new opaqueBackgroundColor: Color white;
						 bounds: trialRect;
						 openInWorld: formerWorld]
		ifNotNil: [translucentMorph _ TranslucentProgressMorph new setProperty: #morphicLayerNumber toValue: targetMorph morphicLayerNumber - 0.1;
						 bounds: targetMorph boundsInWorld;
						 openInWorld: targetMorph world].
	stageCompleted _ 0.
	safetyFactor _ 1.1.
	"better to guess high than low"
	translucentMorph setProperty: #progressStageNumber toValue: 1.
	totals _ self loadingHistoryDataForKey: 'total'.
	newRatio _ 1.0.
	estimate _ totals size < 2
				ifTrue: ["be a pessimist"
					15000]
				ifFalse: [totals sum - totals max
						/ (totals size - 1 max: 1) * safetyFactor].
	start _ Time millisecondClockValue.
	self forkProgressWatcher.
	[aBlock
		on: ProgressInitiationException
		do: [:ex | ex
				sendNotificationsTo: [:min :max :curr | "ignore this as it is inaccurate"
					curr]]]
		on: ProgressNotification
		do: [:note | 
			note extraParam
				ifNotNil: [self addProgressDecoration: note extraParam].
			stageCompletedString _ (note messageText findTokens: ' ') first.
			stageCompleted _ (stageCompletedString copyUpTo: $:) asNumber.
			cumulativeStageTime _ Time millisecondClockValue - start max: 1.
			prevData _ self loadingHistoryDataForKey: stageCompletedString.
			prevData isEmpty
				ifFalse: [newRatio _ (cumulativeStageTime
								/ (prevData average max: 1)) asFloat].
			"SM keep separate histories for local & remote  
			downloads. This knowledge should be moved to  
			ProjectLoading when it can access the cumulative time."
			stageCompleted = 1
				ifTrue: [ProjectLoading HistoryVariant: ''].
			(stageCompleted = 2
					and: [cumulativeStageTime < 1000])
				ifTrue: [ProjectLoading HistoryVariant: 'LocalFile'.
					stageCompletedString _ stageCompletedString , 'LocalFile'].
			self loadingHistoryAt: stageCompletedString add: cumulativeStageTime.
			translucentMorph setProperty: #progressStageNumber toValue: stageCompleted + 1.
			note resume].
	stageCompleted _ 999! !


!ComplexProgressIndicator class methodsFor: 'as yet unclassified' stamp: 'sm 5/6/2001 16:10'!
historyReset
	History _ nil! !


!ProjectLoading class methodsFor: 'as yet unclassified' stamp: 'sm 5/6/2001 16:45'!
HistoryVariant: variantSuffix
	HistoryVariant _ variantSuffix! !

"Postscript:
clear out ComplexProgressIndicator's old historical data so that we see results sooner"
ComplexProgressIndicator historyReset.
!

